#!/bin/sh
#***********************************************************************#
#                                                                       #
#                               NOTICE                                  #
#                                                                       #
#                  COPYRIGHT 2012 QLOGIC CORPORATION                    #
#                        ALL RIGHTS RESERVED                            #
#                                                                       #
#  This computer program is CONFIDENTIAL  and contains TRADE SECRETS of #
#  QLOGIC CORPORATION.  The  receipt or possession of this program does #
#  not convey any rights to reproduce or disclose  its contents,  or to #
#  manufacture, use, or sell anything that it may describe, in whole or #
#  in part, without the specific written consent of QLOGIC CORPORATION. #
#  Any reproduction of this program without the express written consent #
#  of QLOGIC CORPORATION  is a violation of the copyright laws and  may #
#  subject you to civil liability and criminal prosecution.             #
#                                                                       #
#***********************************************************************#
#set -x

print_Usage() {
		echo ""
		echo "-------- QLogic Corporation -- Firmware Dump Utility --------"
		echo ""
		echo "./ql_dump.sh -l ethX|ALL"
		echo "Display firmware dump information on a \"single\" nic or \"all\" nic/storage adapters"
		echo ""
		echo "./ql_dump.sh -s ethX|ALL"
		echo "Save firmware dump from a \"single\" nic or \"all\" nic/storage adapters"
		echo "firmware dump will be saved in directory \"/opt/QLogic_Corporation/FW_Dumps\""
		echo "Saved file will be in \"driver-name_fw_dump_ethX|hostX_date_time.bin.gz\" format"
		echo ""
		echo "./ql_dump.sh -f ethX|hostX"
		echo "Force firmware dump on a single nic/storage adapters"
		echo "While performing force dump please make sure that fw dump is enabled"
		echo ""
		echo "./ql_dump.sh -c ethX|ALL 1|0"
		echo "Enable(1)/Disable(0) firmware dump on a \"single\" nic or \"all\" nic/storage adapters"
		echo ""
		echo "./ql_dump.sh -m ethX|ALL <Mask-Value>"
		echo "Change the current firmware dump capture mask on a \"single\" nic or \"all\" nic/storage adapters"
		echo "Allowed firmware dump capture mask values are 0x3, 0x7, 0xf, 0x1f, 0x3f, 0x7f, and 0xff"
		echo "Default firmware dump capture mask value is 0x1f"
		echo ""
		echo "where ethX|hostX is any interface name belonging to nic/storage adapters"
		echo ""
		exit 1
}

check_ethtool_option()
{ 
	ethtool -h &>/tmp/ethtool_help_output
	cat /tmp/ethtool_help_output | grep "Get dump flag" &>/dev/null
	if [ $? -eq 0 ];then
		cat /tmp/ethtool_help_output | grep "Set dump flag" &>/dev/null
		if [ $? -eq 0 ];then
			ethtool_option=1
		fi
	fi
	rm -f /tmp/ethtool_help_output
}

err() {
                echo "$@" >&2
		logger -t "${0##*/}[$$]" "$@" &>/dev/null
		if [ $? -eq 0 ];then
			return
		else
			echo "$@"
		fi		
}

verify_options_and_no_of_arg(){

	if [ "$option" == "-l" -o "$option" == "-f" -o "$option" == "-s" ];then
        	option_verify=1
        	if [ $NOA -ne 2 ];then
			echo ""
                	echo "Please pass valid number of arguments"
                	echo "Operation aborted"
                	print_Usage
        	fi
	fi

	if [ "$option" == "-c" -o "$option" == "-m" ];then
        	option_verify=1
         	if [ $NOA -ne 3 ];then
			echo ""
                	echo "Please pass valid number of arguments"
                	echo "Operation aborted"
                	print_Usage
        	fi
	fi

	if [ "$option" == "-h" -o "$option" == "--help" ];then
        	print_Usage
        	exit
	fi

	if [ $option_verify -ne 1 ];then
		echo ""
        	echo "Please use valid options"
        	echo "Operation aborted"
        	print_Usage
	fi

}

verify_arg(){

	if [ "$1" == "ALL" ];then
		if [ "$option" == "-f" ];then
			return 1
		else	
			return 0
		fi
	else
		ethtool -i "$1" &>/dev/null
		if [ $? -eq 0 ];then
			ethtool -i "$1" | grep qlcnic &>/dev/null
			if [ $? -eq 0 ];then
				arg_type="qlcnic"
				return 0
			else
				ethtool -i "$1" | grep netxen_nic &>/dev/null
				if [ $? -eq 0 ];then
					arg_type="netxen_nic"
					return 0
				fi
				ethtool -i "$1" | grep nx_nic &>/dev/null
				if [ $? -eq 0 ];then
					arg_type="nx_nic"
					port_no=`ethtool $1 |grep PHYAD |cut -d':' -f2`
					#echo $port_no
					if [ $port_no -ne 0 ] ;then
						echo ""
						echo "nx_nic: Please provide interface belonging to port number 0"
						echo ""
						exit
					fi
					return 0
				fi
			fi
			return 1
		else
			if [ $qla4xxx -eq 1 -o $qla2xxx -eq 1 ];then
				if [ $option == "-f" ];then
					dump_file=/sys/class/scsi_host/$1/device/fw_dump
					if ! test -f ${dump_file} ; then
						ls /sys/class/scsi_host/$1/fw_state &>/dev/null
						if [ $? -eq 0 ];then
							echo ""
							echo "firmware dump interface is not available for $1"
							echo ""
							exit
						else
							return 1
						fi
					else
						arg_type="storage"
						return 0
					fi
				fi
			fi

			return 1
		fi
				
			
	fi
}

check_drivers(){

	lsmod | grep qlcnic &>/dev/null
	if [ $? -eq 0 ];then
		qlcnic=1
	fi

	lsmod | grep qla2xxx &>/dev/null
	if [ $? -eq 0 ];then
		qla2xxx=1
	fi

	lsmod | grep qla4xxx &>/dev/null
	if [ $? -eq 0 ];then
		qla4xxx=1
	fi

	lsmod | grep netxen_nic &>/dev/null
	if [ $? -eq 0 ];then
		netxen_nic=1
	fi

	lsmod | grep nx_nic &>/dev/null
	if [ $? -eq 0 ];then
		nx_nic=1
	fi

	if [ $nx_nic -eq 1 -a $netxen_nic -eq 1 ];then
		check_nx_nic=`ls /proc/net/nx_nic/ | wc -l`
		if [ $check_nx_nic -eq 0 ];then
			nx_nic=0
		else
			netxen_nic=0
		fi
	fi

	if [ $qlcnic -eq 1 -o $netxen_nic -eq 1 -o $nx_nic -eq 1 -o $qla2xxx -eq 1 -o $qla4xxx -eq 1 ];then
		return 0
	else
		return 1
	fi
}


verify_driver_version() {

	cat /sys/module/"$1"/version &>/dev/null
	if [ $? -eq 0 ];then
		no_dots=`cat /sys/module/"$1"/version | grep -o "\." | wc -l`
		#echo $no_dots
		no_dots=$((no_dots+1))
		loop_for_dot=1
		while [ $loop_for_dot -le $no_dots ]
		do
			figure=`cat /sys/module/"$1"/version | cut -d'.' -f"$loop_for_dot"`
			if [ $loop_for_dot -eq 1 ];then
				Major=$figure
				#echo $Major
			fi

			if [ $loop_for_dot -eq 2 ];then
				Minor=$figure
				#echo $Minor
			fi

			if [ $loop_for_dot -eq 3 ];then
				Subver=$figure
				#echo $Subver
			fi

			if [ $loop_for_dot -eq 4 ];then
				build_no=$figure
			fi

			loop_for_dot=$((loop_for_dot+1))
		done
		return 0
	else
		return 1
	fi

}


verify_versions(){

	if [ $qlcnic -eq 1 ];then
        	verify_driver_version "qlcnic"
        	if [ $? -eq 0 ];then
                	if [ $Major -gt 5 -o $Minor -gt 0 -o $Subver -gt 25 ];then
                        	qlcnic_version=1
				if [ $Major -eq 5 -a $Minor -eq 0 -a $Subver -lt 28 ];then
					qlcnic_ethtool_partial_support=1
				fi
                	fi
        	fi
	fi

	if [ $netxen_nic -eq 1 ];then
        	verify_driver_version "netxen_nic"
        	if [ $? -eq 0 ];then
                	if [ $Major -gt 4 -o $Minor -gt 0 -o $Subver -gt 77 ];then
                        	netxen_version=1
				if [ $Major -eq 4 -a $Minor -eq 0 -a $Subver -lt 80 ];then
					netxen_ethtool_partial_support=1
				fi
                	fi
        	fi
	fi


        if [ $nx_nic -eq 1 ];then
                verify_driver_version "nx_nic"
                if [ $? -eq 0 ];then
                        if [ $Major -gt 4 -o $Minor -gt 0 -o $Subver -gt 579 ];then
                                nx_nic_version=1
                        fi
                fi
        fi

}

Find_bus_for_eth() {

	if [ "$arg_type" == "qlcnic" -o "$arg_type" == "netxen_nic" -o "$arg_type" == "nx_nic" ];then
		eth_dir=`find /sys/devices/pci* -name "$ETH" | grep "/net/$ETH"`
		if [ "$eth_dir" == "" ] ; then
			eth_dir="find /sys/devices/pci* -name "net:$ETH""
			BUS=`$eth_dir | awk -F/ '{ for (i = NF; i > 0; --i) print $i }' | head -2 | grep "0000" | cut -d'.' -f1 | cut -d':' -f2,3`
			return 0
		else
			eth_dir="find /sys/devices/pci* -name "$ETH""
			BUS=`$eth_dir | awk -F/ '{ for (i = NF; i > 0; --i) print $i }' | head -3 | grep "0000" | cut -d'.' -f1 | cut -d':' -f2,3`
			#echo $BUS ; exit
			return 0
		fi
	fi
}


Find_dev_for_nx_nic_eth() {


	if [ "$1" == "nx_nic" -a $nx_nic -eq 1 ];then

		for i in `ls /proc/net/nx_nic/`
		do
			j=0
			match=0
			cat /proc/net/nx_nic/"$i"/stats &>/dev/null
			if [ $? -eq 0 ];then
				eth_nx=`cat /proc/net/nx_nic/"$i"/stats |grep "Interface Name" |cut -d':' -f2`
				while [ $j -lt ${#netxen_eth[@]} -a $match -eq 0 ]
				do
					if [ "$eth_nx" == " ${netxen_eth[$j]}" ] ;then
						match=1
						#echo "Match found"
			       	 		nx_nic_dev[j]="$i"	
					fi
					j=$((j+1))	
				done
			fi
		done
	fi
}

find_interfaces_storage(){

	k=0
	m=0
	for i in `lspci | grep $1 | grep "$2"| cut -d' ' -f 1`
	do
		host=`ls /sys/bus/pci/devices/0000\:"$i"/ | grep host` &>/dev/null
		if [ $? -eq 0 ]; then
			ls /sys/class/scsi_host/$host/device/fw_dump &>/dev/null
			if [ $? -eq 0 ]; then
				if [ "$3" == "qla4xxx" ];then
					qla4xxx_dev[m]="0000:"$i""
					qla4xxx_hosts[k]=$host
					#echo ${qla4xxx_dev[$m]}
					#echo ${qla4xxx_hosts[$k]}
				else
					qla2xxx_dev[m]="0000:"$i""
					qla2xxx_hosts[k]=$host
					#echo ${qla2xxx_dev[$m]}
					#echo ${qla2xxx_hosts[$k]}
				fi
				k=$((k+1))
				m=$((m+1))
			fi
		fi
	done
}


Find_All(){
                if [ "$1" == "qlcnic" ];then
                        ls /sys/bus/pci/devices/0000\:"$i" | grep net &>/dev/null
                        if [ $? -eq 0 ] ; then
                                ls /sys/bus/pci/devices/0000\:"$i"/net/ &>/dev/null
                                if [ $? -eq 0 ] ; then
                                        qlcnic_eth[index]=`ls /sys/bus/pci/devices/0000\:"$i"/net/`
                                        qlcnic_dev[index]="0000:"$i""
                                        index=$((index+1))
                                else
                                        qlcnic_eth[index]=`ls /sys/bus/pci/devices/0000\:"$i"/ | grep net | cut -d':' -f 2`
                                        qlcnic_dev[index]="0000:"$i""
                                        index=$((index+1))
                                fi
                        fi
                fi

                if [ "$1" == "netxen_nic" -o "$1" == "nx_nic" ];then
                        ls /sys/bus/pci/devices/0000\:"$i" | grep net &>/dev/null
                        if [ $? -eq 0 ] ; then
                                ls /sys/bus/pci/devices/0000\:"$i"/net/ &>/dev/null
                                if [ $? -eq 0 ] ; then
                                        netxen_eth[index]=`ls /sys/bus/pci/devices/0000\:"$i"/net/`
                                        netxen_dev[index]="0000:"$i""
                                        index=$((index+1))
                                else
                                        netxen_eth[index]=`ls /sys/bus/pci/devices/0000\:"$i"/ | grep net | cut -d':' -f 2`
                                        netxen_dev[index]="0000:"$i""
                                        index=$((index+1))
                                fi
                        fi
                fi
}

find_interfaces_single_and_all(){

        if [ "$ETH" == "ALL" ];then
                if [ "$option" == "-f" -o $3 == "nx_nic" ];then
                        index=0
                        for i in `lspci | grep "$1" | grep "$2" | cut -d' ' -f 1 | cut -d'.' -f1 | uniq`
                        do
                            	i="$i".0
                                Find_All "$3"
                        done
                else
                        index=0
                        for i in `lspci | grep "$1" | grep "$2" | cut -d' ' -f 1`
                        do
                                Find_All "$3"
                        done
                fi
        else
		if [ "$option" == "-f" -o $3 == "nx_nic" ];then
			index=0
			for i in `lspci | grep "$1" | grep "$2" | cut -d' ' -f 1 | cut -d'.' -f1 | uniq | grep "$4"`
			do
				i="$i".0
				Find_All "$3"
			done
		else
                	index=0
                	for i in `lspci | grep "$1" | grep "$2" | cut -d' ' -f 1 | grep "$4"`
                	do
                        	Find_All "$3"
                	done
        	fi
	fi
}


find_interfaces(){

                if [ $qlcnic -eq 1 -a $qlcnic_version -eq 1 ];then
                        find_interfaces_single_and_all "QLogic" "Ethernet" "qlcnic" "$BUS"
                fi

                if [ $netxen_nic -eq 1 -a $netxen_version -eq 1 ];then
                        find_interfaces_single_and_all "NetXen" "Ethernet" "netxen_nic" "$BUS"
                fi

		if [ $nx_nic -eq 1 -a $nx_nic_version -eq 1 ];then
			find_interfaces_single_and_all "NetXen" "Ethernet" "nx_nic" "$BUS"
		fi

		find_interfaces_storage "QLogic" "Network controller" "qla4xxx"
		find_interfaces_storage "QLogic" "Fibre" "qla2xxx"
}


Display_Dump(){

                echo ""
                echo "$3: $1: $2"
                echo "Firmware dump available           :$4"
                echo "Firmware dump size                :$7"
                echo "Firmware dump enabled             :$5"
                echo "Firmware dump capture mask        :0x$6"
                echo ""
                fw_dump_op=$((fw_dump_op+1))
}

ethtool_display(){

	if [ $ethtool_option -eq 1 ];then
		ethtool -w $1 &>/dev/null
		if [ $? -eq 0 ];then
			PROP1=`ethtool -w $1 | cut -d',' -f1 | cut -d':' -f2`
			PROP1=`printf "%x\n" $PROP1`
			PROP2=`ethtool -w $1 | cut -d',' -f3 | cut -d':' -f2 | cut -d' ' -f2`
			if [ $qlcnic_ethtool_partial_support -eq 1 -a $3 == "qlcnic" -o $netxen_ethtool_partial_support -eq 1 -a $3 == "netxen_nic" ];then
				flag1="Unknown"
			else
				state=`ethtool -w $1 | cut -d',' -f1 | cut -d':' -f2`
				if [ $state -eq 0 ];then
					flag1="No"
				else
					flag1="Yes"
				fi
			fi		
			
                	if [ $PROP2 -ne 0 ] ; then
                		flag="Yes"
                	else
                		flag="No"
                	fi
                	Display_Dump "$1" "$2" "$3" "$flag" "$flag1" "$PROP1" "$PROP2 bytes"
		fi
	fi			
}


ethtool_save(){

	if [ $ethtool_option -eq 1 ];then
		ethtool -w $1 &>/dev/null
		if [ $? -eq 0 ];then
			size=`ethtool -w $1 | cut -d',' -f3 | cut -d':' -f2 | cut -d' ' -f2`
			if [ "$size" -ne 0 ] ; then
				ethtool -w $1 data $DFILE
                                if ! test -s "${DFILE}" ; then
                                	rm ${DFILE}
                                	return
				else
					gzip ${DFILE}
					err "$3-$2-$1: Firmware dump saved to file \"${DFILE}.gz\" "
					fw_dump_op=$((fw_dump_op+1))
					return
				fi
                        else
                        	no_fw_dump=$((no_fw_dump+1))
                        	return
                        fi
		fi
	fi
}

ethtool_force(){

	if [ $ethtool_option -eq 1 ];then
		ethtool -w $1 &>/dev/null
		if [ $? -eq 0 ];then
			size=`ethtool -w $1 | cut -d',' -f3 | cut -d':' -f2 | cut -d' ' -f2`
			if [ "$size" -ne 0 ] ; then
                        	echo ""
                                echo "$3-$2-$1: Please save the firmware dump first"
                                echo "Force firmware dump can't be performed"
                                fw_dump_op=$(( $fw_dump_op + 1 ))
                                return
			else
				if [ $qlcnic_ethtool_partial_support -eq 1 -a $3 == "qlcnic" -o $netxen_ethtool_partial_support -eq 1 -a $3 == "netxen_nic" ];then
					ethtool -W $1 0xdeadfeed &>/dev/null
                              		if [ $? -eq 0 ];then
						echo ""
                                		echo "$3-$2-$1: Force firmware dump performed"
                                		fw_dump_op=$(( $fw_dump_op + 1 ))
                                        	return
                                	fi
				else
					state=`ethtool -w $1 | cut -d',' -f1 | cut -d':' -f2`
					if [ $state -eq 0 ];then
						echo ""
						echo "$3-$2-$1: Firmware dump is not enabled"
						echo "Force firmware dump can't be performed"
						fw_dump_op=$(( $fw_dump_op + 1 ))
						return
					else
						ethtool -W $1 0xdeadfeed &>/dev/null
						if [ $? -eq 0 ];then
							echo ""
							echo "$3-$2-$1: Force firmware dump performed"
							fw_dump_op=$(( $fw_dump_op + 1 ))
							return
						fi
					fi
				fi
			fi
		fi
	fi
}

ethtool_enable(){

	if [ $ethtool_option -eq 1 ];then
		ethtool -w $1 &>/dev/null
		if [ $? -eq 0 ];then
			ethtool -W $1 0xaddfeed &>/dev/null
                        if [ $? -eq 0 ];then
                        	fw_dump_op=$(( $fw_dump_op + 1 ))
                        	return
                        fi
		fi
	fi
}

ethtool_disable(){

        if [ $ethtool_option -eq 1 ];then
                ethtool -w $1 &>/dev/null
                if [ $? -eq 0 ];then
                        ethtool -W $1 0xbadfeed &>/dev/null
                        if [ $? -eq 0 ];then
                                fw_dump_op=$(( $fw_dump_op + 1 ))
                                return
                        fi
                fi
        fi

}

ethtool_change_mask(){

	if [ $ethtool_option -eq 1 ];then
		ethtool -w $1 &>/dev/null
		if [ $? -eq 0 ];then
			ethtool -W $1 ${MASK} &>/dev/null
			if [ $? -eq 0 ];then
				fw_dump_op=$(( $fw_dump_op + 1 ))
				return
			fi
		fi
	fi
}


Display_dump_nic(){

        ethtool -i "$1" &>/dev/null
	#echo $1
        if [ $? -eq 0 ];then
                ethtool -i "$1" | grep "$3" &>/dev/null
                if [ $? -eq 0 ];then
			dev_parse=1
                        cat "$4"/fwdump_level &>/dev/null
                        if [ $? -eq 0 ];then
                                PROP1=`cat "$4"/fwdump_level`
                                PROP1=`printf "%x\n" $PROP1`
                                PROP2=`cat "$4"/fwdump_size`
                                ls "$4"/fwdump_state &>/dev/null
                                if [ $? -eq 0 ];then
                                        PROP3=`cat "$4"/fwdump_state`
                                        if [ $PROP3 -ne 0 ] ; then
                                                flag1="Yes"
                                        else
                                                flag1="No"
                                        fi
                                else
                                        flag1="Unknown"
                                fi
                                if [ $PROP2 -ne 0 ] ; then
                                        flag="Yes"
                                else
                                        flag="No"
                                fi
                                Display_Dump "$1" "$2" "$3" "$flag" "$flag1" "$PROP1" "$PROP2 bytes"
			else
				ethtool_display "$1" "$2" "$3" 
                        fi
                fi
        fi
}


Display_dump_storage(){

	ls /sys/module/$1/parameters/$3 &>/dev/null
	if [ $? -eq 0 ];then
		ls /sys/module/$1/parameters/$2 &>/dev/null
		if [ $? -eq 0 ];then	
			state=`cat /sys/module/$1/parameters/$3`
			if [ $state -eq 0 ];then
				flag1="No"
			else
				flag1="Yes"
			fi
			dump_mask=`cat /sys/module/$1/parameters/$2`
			dump_mask=`printf "%x\n" $dump_mask`
			dev_parse=1
			Display_Dump "" "" "$1" "Unknown" "$flag1" "$dump_mask" "Unknown"
		fi 	
	fi
}


Save_dump_storage(){

	mkdir -p $DFILE_PATH &> /dev/null
	qfwd=/sys/class/scsi_host/$1/device/fw_dump
	dfile=${DFILE_PATH}/"$3"_fw_dump_"$1"_`eval date +%Y%m%d_%H%M%S`.bin
	if ! test -f ${qfwd} ; then
		return
	fi

	echo 1 > ${qfwd}
	cat ${qfwd} > ${dfile}
	echo 0 > ${qfwd}

	if ! test -s "${dfile}" ; then
		rm ${dfile}
		no_fw_dump=$((no_fw_dump+1))
		return
	else
		gzip ${dfile}
		err "$3-$2-$1: Firmware dump saved to file \"${dfile}.gz\" "
		dev_parse=1
		fw_dump_op=$((fw_dump_op+1))
		return
	fi
}


Save_dump_nic(){

        mkdir -p $DFILE_PATH &> /dev/null
        QFWD="$4"/fw_dump
        QFWDS="$4"/fwdump_size
        DFILE=${DFILE_PATH}/"$3"_fw_dump_"$1"_`eval date +%Y%m%d_%H%M%S`.bin
        ethtool -i "$1" &>/dev/null
        if [ $? -eq 0 ];then
                ethtool -i "$1" | grep "$3" &>/dev/null
                if [ $? -eq 0 ];then
			dev_parse=1
			cat "$4"/fwdump_level &>/dev/null
                        if [ $? -ne 0 ];then
                   		ethtool_save "$1" "$2" "$3"
                        else
                                size=`cat ${QFWDS}`
                                if [ "$size" -ne 0 ] ; then
                                        while [ $size -ne 0 ]
                                        do
                                                cat ${QFWD} >> ${DFILE} 2>&1
                                                size=`cat ${QFWDS}`
                                        done

                                        if ! test -s "${DFILE}" ; then
                                                rm ${DFILE}
                                                return
                                        else
						gzip ${DFILE}
                                                err "$3-$2-$1: Firmware dump saved to file \"${DFILE}.gz\" "
                                                fw_dump_op=$((fw_dump_op+1))
                                                return
                                        fi
                                else
                                        no_fw_dump=$((no_fw_dump+1))
                                        return
                                fi
                        fi
                fi
        fi
}

Operation() {

        if [ "$ETH" == "ALL" ];then
                Operation_dump "qlcnic" "$1"
                Operation_dump "netxen_nic" "$1"
                Operation_dump "nx_nic" "$1"
        else
                Operation_dump "$arg_type" "$1"
        fi
}


Operation_dump() {

        index=0
        if [ "$1" == "netxen_nic" -a $netxen_version -eq 1 -a $netxen_nic -eq 1 ];then
                while [ $index -lt ${#netxen_eth[@]} ]
                do
                        "$2"_dump_nic ${netxen_eth[$index]} ${netxen_dev[$index]} "netxen_nic" "/sys/class/net/"${netxen_eth[$index]}"/device/"
                        index=$((index+1))
                done
        fi

        if [ "$1" == "qlcnic" -a $qlcnic_version -eq 1 -a $qlcnic -eq 1 ];then
                while [ $index -lt ${#qlcnic_eth[@]} ]
                do
                       	"$2"_dump_nic ${qlcnic_eth[$index]} ${qlcnic_dev[$index]} "qlcnic" "/sys/class/net/"${qlcnic_eth[$index]}"/device/"
                       	index=$((index+1))
                done
        fi

        if [ "$1" == "nx_nic" -a $nx_nic_version -eq 1 -a $nx_nic -eq 1 ];then
                while [ $index -lt ${#nx_nic_dev[@]} ]
                do
                        "$2"_dump_nic ${netxen_eth[$index]} ${netxen_dev[$index]} "nx_nic" "/proc/net/nx_nic/"${nx_nic_dev[$index]}"/"
                        index=$((index+1))
                done
        fi

	if [ "$1" == "qla2xxx" -a $qla2xxx -eq 1 ];then
		while [ $index -lt ${#qla2xxx_hosts[@]} ]
		do
			"$2"_dump_storage ${qla2xxx_hosts[$index]} ${qla2xxx_dev[$index]} "qla2xxx"
			index=$((index+1))
		done
	fi

        if [ "$1" == "qla4xxx" -a $qla4xxx -eq 1 ];then
                while [ $index -lt ${#qla4xxx_hosts[@]} ]
                do
                        "$2"_dump_storage ${qla4xxx_hosts[$index]} ${qla4xxx_dev[$index]} "qla4xxx"
                        index=$((index+1))
                done
        fi  	
}


Save() {

        ls /lib/udev/qlcnic_udev.sh &>/dev/null
        if [ $? -eq 0 ];then
        	ls /etc/udev/rules.d/99-qlcnic.rules &>/dev/null
        	if [ $? -eq 0 ];then
			qlcnic=0
			no_fw_dump=$((no_fw_dump+1))
        	fi
	fi

	ls /lib/udev/qla4xxx_udev.sh &>/dev/null
	if [ $? -eq 0 ];then
		ls /etc/udev/rules.d/99-qla4xxx.rules &>/dev/null
		if [ $? -eq 0 ];then
			qla4xxx=0
			no_fw_dump=$((no_fw_dump+1))
		fi
	fi

        ls /lib/udev/qla2xxx_udev.sh &>/dev/null
        if [ $? -eq 0 ];then
                ls /etc/udev/rules.d/99-qla2xxx.rules &>/dev/null
                if [ $? -eq 0 ];then
                        qla2xxx=0
                        no_fw_dump=$((no_fw_dump+1))
                fi
        fi

	Operation "Save"
        Operation_dump "qla2xxx" "Save"
        Operation_dump "qla4xxx" "Save"

        if [ $no_fw_dump -ne 0 -a $fw_dump_op -eq 0 ];then
                echo "Firmware dump is not available"
                echo ""
                exit
	fi
}

CheckDrvrs_VerifyArgs_and_version() {

	check_drivers

	if [ $? -ne 0 ];then
        	echo "No drivers found"
        	echo "Please load the supported drivers"
        	exit
	fi

	verify_arg $ETH

	if [ $? -ne 0 ];then
        	echo "Invalid argument \"$ETH\""
        	echo "Make sure \"$ETH\" is valid and available driver interface"
        	exit
	fi

	verify_versions
}


Force_dump_nic() {

        ethtool -i "$1" &>/dev/null
        if [ $? -eq 0 ];then
                ethtool -i "$1" | grep "$3" &>/dev/null
                if [ $? -eq 0 ];then
			dev_parse=1
			cat "$4"/fwdump_level &>/dev/null
			if [ $? -eq 0 ];then
				ls "$4"/fwdump_size &>/dev/null
				if [ $? -eq 0 ];then
				size=`cat "$4"/fwdump_size`
				if [ "$size" -ne 0 ] ; then
					echo ""
					echo "$3-$2-$1: Please save the firmware dump first"
					echo "Force firmware dump can't be performed"
					fw_dump_op=$(( $fw_dump_op + 1 ))
					return
				else
					ls "$4"/fwdump_state &>/dev/null
					if [ $? -eq 0 ];then   
						DS=`cat "$4"/fwdump_state`
						if [ $DS -ne 1 ] ; then
							echo ""
							echo "$3-$2-$1: Firmware dump is not enabled"
							echo "Force firmware dump can't be performed"
							fw_dump_op=$(( $fw_dump_op + 1 ))
							return
						else
							echo ""
							echo 0xdeadfeed > "$4"/fw_dump 2>/dev/null
							if [ $? -eq 0 ];then
								echo "$3-$2-$1: Force firmware dump performed"
								fw_dump_op=$(( $fw_dump_op + 1 ))
								return
							fi
						fi
					else
						echo ""
						echo 0xdeadfeed > "$4"/fw_dump 2>/dev/null
						if [ $? -eq 0 ];then
							echo "$3-$2-$1: Force firmware dump performed"
							fw_dump_op=$(( $fw_dump_op + 1 ))
							return
						fi
					fi
				fi
				fi
			else
				ethtool_force "$1" "$2" "$3"
			fi
		fi
	fi
}


Enable_dump_nic() {

	ethtool -i "$1" &>/dev/null
	if [ $? -eq 0 ];then
		ethtool -i "$1" | grep "$3" &>/dev/null
		if [ $? -eq 0 ];then
			dev_parse=1
			cat "$4"/fwdump_level &>/dev/null
			if [ $? -eq 0 ];then
				ls "$4"/fwdump_size &>/dev/null
				if [ $? -eq 0 ];then
					echo 0xaddfeed > "$4"/fw_dump 2>/dev/null
					if [ $? -eq 0 ];then
						fw_dump_op=$(( $fw_dump_op + 1 ))
						return
					fi
				fi
			else
				ethtool_enable "$1"
			fi
		fi
	fi
}

Disable_dump_nic() {

        ethtool -i "$1" &>/dev/null
        if [ $? -eq 0 ];then
                ethtool -i "$1" | grep "$3" &>/dev/null
                if [ $? -eq 0 ];then
			dev_parse=1
                     	cat "$4"/fwdump_level &>/dev/null
                        if [ $? -eq 0 ];then
				ls "$4"/fwdump_size &>/dev/null
				if [ $? -eq 0 ];then
                                	echo 0xbadfeed > "$4"/fw_dump 2>/dev/null
					if [ $? -eq 0 ];then
                                		fw_dump_op=$(( $fw_dump_op + 1 ))
                                		return
					fi
				fi
			else
				ethtool_disable "$1"
                        fi
                fi
        fi
}

verify_mask() {

	hex_prefix=`echo "$1" | cut -c 1-2`
	if [ "$hex_prefix" == "0x" ];then
		printf "%d\n" $1 &>/dev/null
		if [ $? -eq 0 ];then
			mask=`printf "%d\n" $1` &>/dev/null
			case $mask in
		 	3) ;;
			7) ;;
       	       	       15) ;;
               	       31) ;;
               	       63) ;;
                      127) ;;
                      255)
			   invalid_mask_for_storage=1
			   ;;
	                *) echo "Invalid capture mask "\"$MASK"\""
	                   echo "Operation aborted"
	                   echo ""
	                   exit;;
	                esac
		else
			echo "\"$1\" is not a valid hex number"
			echo ""
			exit
		fi
	else
		echo "Please provide mask values in hexa-decimal format number"
		echo "for example: \"0xValue\""
		echo ""
		exit
	fi
}

ChangeMask_dump_nic() {

        ethtool -i "$1" &>/dev/null
        if [ $? -eq 0 ];then
                ethtool -i "$1" | grep "$3" &>/dev/null
                if [ $? -eq 0 ];then
			dev_parse=1
                        cat "$4"/fwdump_level &>/dev/null
                        if [ $? -eq 0 ];then
                                echo ${MASK} > "$4"/fwdump_level 2>/dev/null
				if [ $? -eq 0 ];then
                                	fw_dump_op=$(( $fw_dump_op + 1 ))
                                	return
				fi
			else
				ethtool_change_mask "$1"
                        fi
                fi
        fi

}

Change_storage_module_parameter(){

	if [ $1 == "qla4xxx" -a $2 == "ql4xenablemd" ];then
		ls /sys/module/qla4xxx/parameters/ql4xenablemd &>/dev/null
		if [ $? -eq 0 ];then
			echo $3 > /sys/module/qla4xxx/parameters/ql4xenablemd
			dev_parse=1
			fw_dump_op=$(( $fw_dump_op + 1 ))
		fi
		return
	fi 

	lsmod | grep "$1" &>/dev/null
	if [ $? -eq 0 ];then
		rmmod $1 &>/dev/null
		if [ $? -ne 0 ];then
			return
		else
			modprobe $1 $2=$3 &>/dev/null
			if [ $? -eq 0 ];then
				dev_parse=1
				fw_dump_op=$(( $fw_dump_op + 1 ))
			fi
			return
		fi
	fi
}

Force_dump_storage(){

	if test -d /sys/class/fc_host/$ETH ; then
		ls /sys/module/qla2xxx/parameters/ql2xmdenable &>/dev/null
		if [ $? -eq 0 ];then
			dump_state=`cat /sys/module/qla2xxx/parameters/ql2xmdenable`
			driver_name=qla2xxx
		else
			return
		fi
	else
		ls /sys/module/qla4xxx/parameters/ql4xenablemd &>/dev/null
		if [ $? -eq 0 ];then
			dump_state=`cat /sys/module/qla4xxx/parameters/ql4xenablemd`
			driver_name=qla4xxx
		else
			return
		fi
	fi

	if [ $dump_state -eq 1 ];then
		echo 2 > $dump_file
		echo 3 > $dump_file
		echo "$driver_name-$ETH: Force firmware dump performed"
	else
		echo "$driver_name-$ETH: Firmware dump is not enabled"
                echo "Force firmware dump can't be performed"
		
	fi

        dev_parse=1
        fw_dump_op=$(( $fw_dump_op + 1 ))
}
	

option=$1
ETH=$2
MASK=$3
DFILE_PATH=/opt/QLogic_Corporation/FW_Dumps
NOA=$#
option_verify=0
netxen_nic=0
qlcnic=0
nx_nic=0
qla2xxx=0
qla4xxx=0
netxen_version=0
qlcnic_version=0
nx_nic_version=0
arg_type=""
BUS=""
fw_dump_op=0
no_fw_dump=0
dev_parse=0
build_no=""
ethtool_option=0
qlcnic_ethtool_partial_support=0
netxen_ethtool_partial_support=0
invalid_mask_for_storage=0

# Declares An Array in which All the interface Names and device names will be stored
declare -a netxen_eth
declare -a netxen_dev
declare -a qlcnic_eth
declare -a qlcnic_dev
declare -a nx_nic_dev
declare -a qla2xxx_hosts
declare -a qla2xxx_dev
declare -a qla4xxx_hosts
declare -a qla4xxx_dev

if [ "$#" == 0 ] ; then
	print_Usage
fi

verify_options_and_no_of_arg
CheckDrvrs_VerifyArgs_and_version

if [ "$ETH" != "ALL" ];then
	Find_bus_for_eth
fi

find_interfaces
Find_dev_for_nx_nic_eth "nx_nic"


check_ethtool_option

case "$option" in
-l)
	echo ""
	echo "<----- Displaying firmware dump information ----->"
	echo ""
	Operation "Display"
	if [ "$ETH" == "ALL" ];then
		Display_dump_storage "qla2xxx" "ql2xmdcapmask" "ql2xmdenable"
		Display_dump_storage "qla4xxx" "ql4xmdcapmask" "ql4xenablemd"
	fi

	echo ""
	;;
-s)
	echo ""
	echo "<----- Saving the firmware dump ----->"
	echo ""
	Save
	echo ""
	;;
-f)
	echo ""
	echo "<----- Forcing the firmware dump ----->"
	echo ""

	if [ "$arg_type" == "storage" ];then
		Force_dump_storage
	fi

	Operation "Force"
	echo ""
	;;
-c)
	echo ""
	echo "<----- Changing the firmware dump capability ----->"
	if [ "$3" == "0" -o "$3" == "1" ] ; then
		if [ $3 -ne 0 ] ; then
			echo ""
			Operation "Enable"
			if [ "$ETH" == "ALL" ];then
				Change_storage_module_parameter "qla2xxx" "ql2xmdenable" "1"
				Change_storage_module_parameter "qla4xxx" "ql4xenablemd" "1"
			fi
		else
			echo ""
			Operation "Disable"
			if [ "$ETH" == "ALL" ];then
				Change_storage_module_parameter "qla2xxx" "ql2xmdenable" "0"
				Change_storage_module_parameter "qla4xxx" "ql4xenablemd" "0"
			fi
		fi
	else
		echo ""
		echo "Please check the argument \"$3\""
		echo "Operation aborted"
		print_Usage
	fi

	if [ $fw_dump_op -ne 0 ] ;then
		if [ $3 -ne 0 ];then
			echo "Firmware dump is enabled"
			echo ""
			exit
		else
			echo "Firmware dump is disabled"
			echo ""
			exit
		fi
	fi
	;;
-m)
	echo ""
	echo "<----- Changing the firmware dump capture mask value ----->"
	echo ""
	verify_mask $MASK
	Operation "ChangeMask"

	if [ $invalid_mask_for_storage -eq 0 -a "$ETH" == "ALL" ];then
		Change_storage_module_parameter "qla2xxx" "ql2xmdcapmask" "$MASK"
		Change_storage_module_parameter "qla4xxx" "ql4xmdcapmask" "$MASK"
	fi

	if [ $fw_dump_op -ne 0 ] ;then
		echo "Firmware dump mask is changed to \"${MASK}\""
		echo ""
		exit
	fi
	;;
*)
	print_Usage
	;;
esac


if [ $fw_dump_op -eq 0 ];then
       	if [ $qlcnic_version -eq 0 -a $netxen_version -eq 0 -a $nx_nic_version -eq 0 ];then
               	echo ""
		echo "Operation is not supported for current drivers"
               	echo ""
                echo "\"qlcnic\" driver requires driver version 5.0.26 or higher"
                echo "\"netxen_nic\" driver requires driver version 4.0.78 or higher"
                echo "\"nx_nic\" driver requires driver version 4.0.580 or higher"
		echo ""
               	exit
	else
		if [ $dev_parse -eq 0 ];then
			echo ""
			echo "No drivers/fw dump interfaces available for this operation"
			echo ""
			exit
		else
			echo ""
			echo "No interface available to drivers for this operation"  
			echo ""
			exit
		fi
       	fi
fi
